////////////////////////////////////////////////////////////////
//      NOTE: This is the combination of HRroad.h and         //
//                 HRroad.cpp                                 //
////////////////////////////////////////////////////////////////

// This file is created for a programming class project
// Yes, I should learn to use STL sometime
#include <iostream.h>
#include <assert.h>

template <class T>
class List;
// Item<T> is the basic unit for a List<T>
// Item<T> object includes 2 pointers: prev and next
// Item<T>->getval() returns the real stuff stores in the Item
//         which is type T
template <class T>
class Item {
        friend class List<T>;
public:
        T getval() { return val; }
        void setval(T value) {val = value;}
protected:
        Item(T value = 0, Item *prevItem = 0, Item *nextItem = 0) {
                val = value; prev = prevItem; next = nextItem;
        }
        T val;
        Item<T> *prev, *next;
};

// List basically is a weakly based set.
// List enable user to append, pop and retrieve
// Items. reset() must be called each time before
// retrieve all members of the List
template <class T>
class List {
public:
        List() { first = last = cursor = 0; }
        ~List();
        void reset() { cursor = first; }
        T pop();
        void append(T);
        Item<T> *retrieve();
        void display();
        int is_empty();
private:
        Item<T> *first, *last, *cursor;
};

template <class T>
void List<T>::append(T value) {
        Item<T> *pt = new Item<T>(value);
        assert(pt != 0);
        if (first == 0) //empty list
                first = pt;
        else {
                last->next = pt;
                pt ->prev = last;
        }
        last = pt;
}

template <class T>
T List<T>::pop() {
        if (first == 0) return 0; //empty list
        Item<T> *tmp = first;
        first = first->next;
        if (first != 0) { first->prev = 0;}
        tmp->next = 0;
        return tmp->getval(); //poped element is not deleted from memory,
                        // no need to make another copy.
}

template <class T>
void List<T>::display() {
        cout << "[ ";
// << first << ' ' << last << ' ';
        for (Item<T> *pt = first; pt; pt = pt->next)
                cout << (pt->val) << ' ';
/*  << pt->prev << ' ' << pt->next */
        cout << "]";
}

// destructor take care of the memories located by all list items
template <class T>
List<T>::~List() {
        Item<T> *pt = first;
        while (pt) {
                Item<T> *tmp = pt;
                pt = pt->next;
                delete tmp;
        }
        first = last = 0;
}

// retrieve traverses the list member one by one
template <class T>
Item<T> *List<T>::retrieve() {
        Item<T> *tmp = cursor;
        if (cursor != 0)
                cursor = cursor->next;
        return tmp; //return 0 when cursor reach the end
}

template <class T>
int List<T>::is_empty() {
        return (first == 0);
}

template <class T>
class Set_s;
// Item_s means: Item for Strongly Based Set
// Item_s is different from Item mainly by its membership
// field and all pointers are essentially integers, because
// memebers are in an array
template <class T>
class Item_s{
        friend class Set_s<T>;
public:
        Item_s(int prevItem = -1, int nextItem = -1) {
                prev = prevItem; next = nextItem; membership = 0;
        }
protected:
        int prev, next;
        int membership;
        T       val;
};

// Set_s:  Strongly Based Class, an array of Item_s
// enable random access, random add 
template <class T>
class Set_s {
public:
        Set_s(int);
        ~Set_s();
        int is_member(int);
        int is_empty();
        void reset() { cursor = first; }
        int retrieve();
        void setValue(int index, T value) { stuff[index].val = value; }
        T& getValue(int index) { return stuff[index].val; }
//      ^^^^ modification make during programming
        void add(int);
        void remove(int);
        void display();
        void increase(int); // increase membership value
        void decrease(int); // decrease membership value
// membership field is not limited to 0/1 because Numpred Set_s
// used later needs range of 0..max number of vertices
private:
        int size, first, last, cursor;
        Item_s<T> *stuff;
};

template <class T>
Set_s<T>::Set_s(int sz = 0) {
        size = sz;
        first = last = cursor = -1;
        stuff = 0;
        if (size == 0) return;

        stuff = new Item_s<T>[size];
        assert(stuff != 0);
}

// delocate memor
template <class T>
Set_s<T>::~Set_s() {
        delete [] stuff;
}

template <class T>
int Set_s<T>::is_empty() {
        return ((first < 0) || (stuff[first].membership == 0));
}

template <class T>
void Set_s<T>::increase(int index) {
        assert(0 <= index && index < size);
        stuff[index].membership++;
}

template <class T>
void Set_s<T>::decrease(int index) {
        assert(0 <= index && index < size);
        stuff[index].membership--;
}

template <class T>
void Set_s<T>::add(int index) {
        assert(0 <= index && index < size);
        if (is_empty())
                first = index;
        else {
                stuff[last].next = index;
                stuff[index].prev = last;
        }
        last = index;
        stuff[last].membership = 1;
}

template <class T>
int Set_s<T>::retrieve() {
        if (cursor >= 0) {
                int tmp = cursor;
                cursor = stuff[cursor].next;
    return tmp;
        }
                else return -1;
        //return -1 when cursor reach the end
}

template <class T>
void Set_s<T>::remove(int index) {
        assert(0 <= index && index < size);
// assume element be a member
        if (index == first) // the first element
                if (first == last) {// only one element
                        first = last = -1;
                }
                else {
                        first = stuff[index].next;
                        stuff[first].prev = -1;
                }
        else
                if (index == last) {
                        int tmp = stuff[last].prev;
                        stuff[tmp].next = -1;
                        last = tmp;
                }
                else {
                        int tmp1 = stuff[index].prev;
                        int tmp2 = stuff[index].next;
                        stuff[tmp1].next = tmp2;
                        stuff[tmp2].prev = tmp1;
                }
        stuff[index].prev = stuff[index].next = -1;
        stuff[index].membership = 0;
}

template <class T>
int Set_s<T>::is_member(int index) {
        assert(0 <= index && index < size);
        return (stuff[index].membership);
}

template <class T>
void Set_s<T>::display() {
        cout << "{ "; // << first << ' ' << last << ' ';
        for (int i = 0; i < size; i++)
                cout << stuff[i].membership << ' ';
                 //     << stuff[i].prev << ' '
                /*      << stuff[i].next */
        cout << "}\n";
}

template <class T1, class T2>
class Mmap_b;
//Item_b: based Item(weakly) used for weakly based mmap
// domain is type T1 while range is type T2
template <class T1, class T2>
class Item_b:public Item<T1> {
        friend class Mmap_b<T1, T2>;
public:
        Item_b(T1 value = 0):Item<T1>(value) {
                ImageSet = 0;
        }
        ~Item_b();
        List<T2> *getImage() { return ImageSet; }
// getImage provide the access to the Image list of a certain domain
private:
        List<T2> *ImageSet;
};

template <class T1, class T2>
Item_b<T1, T2>::~Item_b() {
        delete ImageSet;
}

template <class T1, class T2>
class Mmap_b {
public:
        Mmap_b();
        ~Mmap_b();
        Item_b<T1, T2> *append(Item_b<T1, T2> *, T2);
        Item_b<T1, T2> *append(T1 domain, T2 range);
        void reset();
        Item_b<T1, T2> * retrieve();
        void remove(Item_b<T1, T2> *);
        void display();
private:
        Item_b<T1, T2> *first, *last, *cursor;
};

// Mmap_b: weakly based multimap mmap[B,B]
template <class T1, class T2>
void Mmap_b<T1, T2>::reset() {
        cursor = first;
}

template <class T1, class T2>
Item_b<T1, T2> *Mmap_b<T1, T2>::retrieve() {
        Item_b<T1, T2> *tmp = cursor;
        if (cursor != 0)
                cursor = (Item_b<T1, T2> *)cursor->next;
        return tmp; //return 0 when cursor reach the end
}

template <class T1, class T2>
Mmap_b<T1, T2>::Mmap_b() {
        first = last = cursor = 0;
}

template <class T1, class T2>
Mmap_b<T1, T2>::~Mmap_b() {
        Item_b<T1, T2> *pt = first;
        while (pt) {
                Item_b<T1, T2> *tmp = pt;
                pt = (Item_b<T1, T2> *) pt->next;
//              delete tmp->ImageSet;
                delete tmp;
        }
}

// this append only allow insert pairs as an independent pair,
// which means no check for domain duplication
template <class T1, class T2>
Item_b<T1, T2> *Mmap_b<T1, T2>::append(T1 domain, T2 range) {
// Assume domain is a new one
        Item_b<T1, T2> *pt = new Item_b<T1, T2>(domain);
        assert(pt != 0);
        if (first == 0) //This is the first domain entry
                first = pt;
        else {
                last->next = pt;
                pt->prev = last;
        }
        last = pt;
        append(pt, range);
        return pt;
}

// this append enable insert image into an existed domain
template <class T1, class T2>
Item_b<T1, T2> *Mmap_b<T1, T2>::append(Item_b<T1, T2> * domain, T2 range) {
// Assume domain has already been in the mmap
        if (domain->ImageSet == 0) domain->ImageSet = new List<T2>;
        assert(domain->ImageSet != 0);
        domain->ImageSet->append(range);
  return domain;
}


template <class T1, class T2>
void Mmap_b<T1, T2>::remove(Item_b<T1, T2> * item) {
        if (item == first) {
                if (first == last) { //only one element
                        first = last = 0;
                } else {
                        first = (Item_b<T1, T2> *) first->next;
                        first->prev = 0;
                }
        } else
                if (item == last) {
                        last = (Item_b<T1, T2> *) last->prev;
                        last->next = 0;
                } else {
                        ((Item_b<T1, T2> *)(item->prev))->next = item->next;
                        ((Item_b<T1, T2> *)(item->next))->prev = item->prev;
                }
        delete item;
}

template <class T1, class T2>
void Mmap_b<T1, T2>::display() {
        cout << "{"; // << first << ' ' << last << ' ';
        for (Item_b<T1, T2> *pt = first; pt; pt = (Item_b<T1, T2> *)pt->next) {
                cout << '[' << pt->val << ',';
                         pt->ImageSet->display();
                         cout << ']';
        }
        cout << "}\n";
}

template <class T1, class T2>
class Mmap_s;
// Item_sb: Item for Strongly Based Mmap, here pointers are
// essentially intergers
template <class T1, class T2>
class Item_sb:private Item_s<T1> {
        friend class Mmap_s<T1, T2>;
public:
        Item_sb():Item_s<T1>() {
                ImageSet = 0;
        }
        ~Item_sb();
private:
        List<T2> *ImageSet;
};

template <class T1, class T2>
Item_sb<T1, T2>::~Item_sb() {
        delete ImageSet;
}

// Strongly Based MMap mmap[B-S, B]
// domain is type T1, range is type T2, pointers are intergers
// because all members are in an array
template <class T1, class T2>
class Mmap_s {
public:
        Mmap_s(int);
        ~Mmap_s();
        int is_member(int);
        int is_empty();
        void add(int, T2);
        List<int> *getImage(int);
        void display();
private:
        int size, first, last;
        Item_sb<int, int> *stuff;
};

template <class T1, class T2>
Mmap_s<T1, T2>::Mmap_s(int sz = 0) {
        size = sz;
        first = last = -1;
        stuff = 0;
        if (size == 0) return;

        stuff = new Item_sb<T1, T2>[size];
        assert(stuff != 0);
}

template <class T1, class T2>
Mmap_s<T1, T2>::~Mmap_s() {
        delete [] stuff;
}

template <class T1, class T2>
List<T2> *Mmap_s<T1, T2>::getImage(int index) {
        assert(0 <= index && index < size);
        return stuff[index].ImageSet;
}

template <class T1, class T2>
int Mmap_s<T1, T2>::is_empty() {
        return ((first < 0) || (stuff[first].membership == 0));
}

template <class T1, class T2>
void Mmap_s<T1, T2>::add(int index, T2 range) {
        assert(0 <= index && index < size);
        if (stuff[index].membership == 0) { // a new entry
                if (is_empty()) first = index;
                else {
                        stuff[last].next = index;
                        stuff[index].prev = last;
                }
                stuff[index].membership = 1;
                last = index;
                // now add the range
                stuff[index].ImageSet = new List<T2>;
                assert(stuff[index].ImageSet != 0);
        }
         // entry already there
        stuff[index].ImageSet->append(range);
}

template <class T1, class T2>
int Mmap_s<T1, T2>::is_member(int index) {
        assert(0 <= index && index < size);
        return (stuff[index].membership);
}

template <class T1, class T2>
void Mmap_s<T1, T2>::display() {
        cout << "( " << first << ' ' << last << ' ';
        for (int i = 0; i < size; i++) {
                cout << '[' << stuff[i].membership << ',';
//                      << stuff[i].prev << ' '
//                      << stuff[i].next << ',';
                         stuff[i].ImageSet->display();
                         cout << ']';
        }
        cout << ")\n";
}

////////////////////////////
template <class T>
class Hashtable {
        List<T *> **hash;
        long size;
public:
        long usedSize;
        long nofElm;
        T **elm;
        Hashtable(long sz, long nofS) {
                size = sz;
                hash = new (List<T *> *);//[size];
                for (int i = 0; i < sz; i++)
                        hash[i] = new List<T *>;
                usedSize = 0;
                nofElm = 0;
                elm = new (T *);//[nofS];
        }
        ~Hashtable() {
                delete [] hash;
                delete [] elm;
        }
        long put(T *obj);
        T *get(T *obj);
};

template <class T>
long Hashtable<T>::put(T *obj) {
        List<T *> *list = hash[obj->hashCode() % size];
        if (list->is_empty())
                usedSize++;
        list->append(obj);
        elm[nofElm++] = obj;
        return nofElm-1;
}

template <class T>
T *Hashtable<T>::get(T *obj) {
        List<T *> *list = hash[obj->hashCode() % size];
        list->reset();
        Item<T *> *tmpItem;
                while ((tmpItem = list->retrieve()) != 0) {
                        T *tmp = tmpItem->getval();
                        if (tmp->equal(obj))
                                return tmp;
                }
        return NULL;
}

//
//      Game Solver  By Wei Wang   10/29/97
//      Last modified 04/10/98 by Wei Zhang 
//
#include <iostream.h>
//This is my own STL!
#include "huarong.h"

//Allocate an array using 10 integers, ugly but handy
int *newArray(int i0, int i1, int i2, int i3, int i4, int i5, int i6, int i7, int i8, int i9) {
        int *ar = new int[10];
        ar[0] = i0; //the big square
        ar[1] = i1; //the only wide piece
        ar[2] = i2; //four long pieces
        ar[3] = i3;
        ar[4] = i4;
        ar[5] = i5;
        ar[6] = i6; //four little squares
        ar[7] = i7;
        ar[8] = i8;
        ar[9] = i9;
        return ar;
}

//State remembers the position of each piece
/*Coordinate System
  0  1  2  3
  4  5  6  7
  8  9  10 11
  12 13 14 15
  16 17 18 19
*/
class State {
public:
        //nof steps to reach this state from initial state
        int steps;
        //piece array, store the position of the upper left corner of a piece
        int* p;
        //if a position is occupied or not
        int occupy[20];
        //the best parent state for traveling
        State* parent;
        //all the reachable childrens
        List<State*> *childS;
        State(int pp[]);
        int order;

        //generate all reachable states according to the rule
        List<State*> *generateNextStates();

        //finished?
        int done() {
                //if Cao-cao comes to the EXIT
                return (p[0] == 13);
        }

        //print out
        void show();

        int hashCode();

        int equal(State *);
        //generate legal moves for the four little squares
        void moveZ(List<State*> *sq, int z, int r2, int r3, int r4);
        //generate legal moves for the four long pieces
        void moveJ(List<State*> *sq, int j, int r2, int r3, int r4);
};

        void State::show() {
                for (int i = 0; i < 10; i++)
                        cout << p[i] << ' ';
                cout << endl;
        }

State::State(int pp[])
        {
                steps = 9999; //a large number means require
                // infinite steps to reach this state//MAXINT; // cannot reach the solution
                parent = NULL;
                childS = NULL;
                p = pp;
                int J[4], Z[4], i, j;
                //different orders of Js and Zs are the same state, so we sort them
                //sort p[2], p[3], p[4], p[5] and p[6], p[7], p[8], p[9].
                for (i = 0; i < 4; i++) {
                        int index = 0;
                        for (j = 0; j < 4; j++) {
                                if (p[2+i] > p[2+j])
                                        index++;
                        }
                        J[index] = p[2+i];
                }
                for (i = 0; i < 4; i++) {
                        int index = 0;
                        for (j = 0; j < 4; j++) {
                                if (p[6+i] > p[6+j])
                                        index++;
                        }
                        Z[index] = p[6+i];
                }
                for (i = 0; i < 4; i++) {
                        p[2+i] = J[i];
                        p[6+i] = Z[i];
                }
        for (i = 0; i < 20; i++)
                occupy[i] = 0;
        occupy[p[0]] = occupy[p[0]+1] = occupy[p[0]+4] = occupy[p[0]+5] = 1;
        occupy[p[1]] = occupy[p[1]+1] = 1;
        occupy[p[6]] = occupy[p[7]] = occupy[p[8]] = occupy[p[9]] = 1;
        occupy[p[2]] = occupy[p[2]+4] = occupy[p[3]] = occupy[p[3]+4] = 1;
        occupy[p[4]] = occupy[p[4]+4] = occupy[p[5]] = occupy[p[5]+4] = 1;
        }

int State::hashCode(){
        //ad hoc hashing methods, not bad, 4 states hashed to the same bucket by average
        int code = p[0];
        for (int i = 1; i < 10; i++) {
                code <<= 1;
                code ^= p[i];
        }
        code &= 0xFFFF;
        return code;
}

int State::equal(State *s){
        //if two states are the same?
        for (int i = 0; i < 10; i++) {
                if (p[i] != s->p[i])
                        return 0;
        }
        return 1;
}

void State::moveZ(List<State*> *sq, int z, int r2, int r3, int r4) {
        int x = z % 4; int y = z / 4;
        //move 2 pos left
        if (x > 1 && occupy[z-2] == 0 && occupy[z-1] == 0)
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-2,r2,r3,r4)));
        //move 1 pos left
        if (x > 0 && occupy[z-1] == 0) {
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-1,r2,r3,r4)));
                //move 1 down
                if (y < 4 && !occupy[z+3])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+3,r2,r3,r4)));
                //move 1 up
                if (y > 0 && !occupy[z-5])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-5,r2,r3,r4)));

        }
        //move 2 right
        if (x < 2 && occupy[z+1] == 0 && occupy[z+2] == 0)
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+2,r2,r3,r4)));
        //move 1 right
        if (x < 3 && occupy[z+1] == 0) {
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+1,r2,r3,r4)));
                //move 1 down
                if (y < 4 && !occupy[z+5])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+5,r2,r3,r4)));
                //move 1 up
                if (y > 0 && !occupy[z-3])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-3,r2,r3,r4)));
        }
        //move 2 down
        if (y < 3 && occupy[z+4] == 0 && occupy[z+8] == 0)
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+8,r2,r3,r4)));
        //move 1 down
        if (y < 4 && occupy[z+4] == 0) {
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+4,r2,r3,r4)));
                //move 1 left
                if (x > 0 && !occupy[z+3])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+3,r2,r3,r4)));
                //move 1 right
                if (x < 3 && !occupy[z+5])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z+5,r2,r3,r4)));
        }
        //move 2 up
        if (y > 1 && occupy[z-4] == 0 && occupy[z-8] == 0)
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-8,r2,r3,r4)));
        //move 1 up
        if (y > 0 && occupy[z-4] == 0) {
                sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-4,r2,r3,r4)));
                //move 1 left
                if (x < 3 && !occupy[z-3])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-3,r2,r3,r4)));
                //move 1 right
                if (x > 0 && !occupy[z-5])
                        sq->append(new State(newArray(p[0],p[1],p[2],p[3],p[4],p[5],z-5,r2,r3,r4)));
        }
}

void State::moveJ(List<State*> *sq, int j, int r2, int r3, int r4) {
        int x = j % 4; int y = j / 4;
        //move 1 left
        if (x > 0 && occupy[j-1] == 0 && occupy[j+3] == 0)
                sq->append(new State(newArray(p[0],p[1],j-1,r2,r3,r4,p[6],p[7],p[8],p[9])));
        //move 1 right
        if (x < 3 && occupy[j+1] == 0 && occupy[j+5] == 0)
                sq->append(new State(newArray(p[0],p[1],j+1,r2,r3,r4,p[6],p[7],p[8],p[9])));
        //move 2 down
        if (y < 2 && occupy[j+8] == 0 && occupy[j+12] == 0)
                sq->append(new State(newArray(p[0],p[1],j+8,r2,r3,r4,p[6],p[7],p[8],p[9])));
        //move 1 down
        if (y < 3 && occupy[j+8] == 0)
                sq->append(new State(newArray(p[0],p[1],j+4,r2,r3,r4,p[6],p[7],p[8],p[9])));
        //move 1 up
        if (y > 0 && occupy[j-4] == 0)
                sq->append(new State(newArray(p[0],p[1],j-4,r2,r3,r4,p[6],p[7],p[8],p[9])));
        //move 2 up
        if (y > 1 && occupy[j-4] == 0 && occupy[j-8] == 0)
                sq->append(new State(newArray(p[0],p[1],j-8,r2,r3,r4,p[6],p[7],p[8],p[9])));
}

List<State*> * State::generateNextStates() {
        List<State*> *sq = new List<State*>;
        childS = sq;
        int x = p[0] % 4; int y = p[0] / 4; //row and column
        //cao moves
        //move left
        if (x > 0 && occupy[p[0]-1] == 0 && occupy[p[0]+3] == 0)
                sq->append(new State(newArray(p[0]-1,p[1],p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move right
        if (x < 2 && occupy[p[0]+2] == 0 && occupy[p[0]+6] == 0)
                sq->append(new State(newArray(p[0]+1,p[1],p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move down
        if (y < 3 && occupy[p[0]+8] == 0 && occupy[p[0]+9] == 0)
                sq->append(new State(newArray(p[0]+4,p[1],p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move up
        if (y > 0 && occupy[p[0]-4] == 0 && occupy[p[0]-3] == 0)
                sq->append(new State(newArray(p[0]-4,p[1],p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //p[1] moves
        x = p[1] % 4; y = p[1] / 4;
        //move 2 left
        if (x > 1 && occupy[p[1]-2] == 0 && occupy[p[1]-1] == 0)
                sq->append(new State(newArray(p[0],p[1]-2,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move 1 left
        if (x > 0 && occupy[p[1]-1] == 0)
                sq->append(new State(newArray(p[0],p[1]-1,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move 2 right
        if (x < 1 && occupy[p[1]+2] == 0 && occupy[p[1]+3] == 0)
                sq->append(new State(newArray(p[0],p[1]+2,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move 1 right
        if (x < 2 && occupy[p[1]+2] == 0)
                sq->append(new State(newArray(p[0],p[1]+1,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move 1 down
        if (y < 4 && occupy[p[1]+4] == 0 && occupy[p[1]+5] == 0)
                sq->append(new State(newArray(p[0],p[1]+4,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //move 1 up
        if (y > 0 && occupy[p[1]-4] == 0 && occupy[p[1]-3] == 0)
                sq->append(new State(newArray(p[0],p[1]-4,p[2],p[3],p[4],p[5],p[6],p[7],p[8],p[9])));
        //j moves
        moveJ(sq, p[2],p[3],p[4],p[5]);
        moveJ(sq, p[3],p[2],p[4],p[5]);
        moveJ(sq, p[4],p[2],p[3],p[5]);
        moveJ(sq, p[5],p[2],p[3],p[4]);
        moveZ(sq, p[6],p[7],p[8],p[9]);
        moveZ(sq, p[7],p[6],p[8],p[9]);
        moveZ(sq, p[8],p[6],p[7],p[9]);
        moveZ(sq, p[9],p[6],p[7],p[8]);
        return sq;
}

Hashtable<State> *hash;

void explore(State *s) {
        //generate all reachable states, for each of these states explore
        //all their reachable sates also, unless a state is the DONE state
        //or a state explored or being explored
        //The purpose is build the Graph:
        //    each reachable state is a vertex
        //    each move connect two states
        //    then use SSSP algorithm to find the shortest path
        List<State *> *sq = s->generateNextStates();
        sq->reset();
        Item<State *> *tmpItem;
        while ((tmpItem = sq->retrieve()) != NULL) {
                State *subS = tmpItem->getval();
                //subS->show();
                State *existenS;
                //if a state is already created then used the existen one
                if ((existenS = hash->get(subS)) != NULL) {
                        //cout << "VISITED\n";
                        //point to existen State 
                        tmpItem->setval(existenS);
                        delete subS;
                        continue;
                }
                //explore a new sub state, mark it as being checked first
                subS->order = hash->put(subS);
                //cout << "NEW " << subS->order << endl;
                if (subS->done()) {
                        cout << "FOUND " << subS->order << endl;
                        subS->show();
                        continue;
                }
                //subS->show();
                //explore a new state
                explore(subS);
        }
}


//print out the path generate by SSSP algorithm
void printSSSP(State *s, State *w) {
        if (s == w) {
                w->show();
                return;
        }
        printSSSP(s, w->parent);
        w->show();
        return;
}

int main() {
        //this is the total nof States found by pre-runs
        const unsigned nofS = 24635;
        hash = new Hashtable<State>(12289, nofS);
        cout << "Finish Init\n";
        //create initState
        State *initS = new State(newArray(1,9,0,3,8,11,13,14,16,19));
        cout << "INIT " << (initS->order = hash->put(initS)) << endl;
        //build the Graph
        explore(initS);
        // output the optimal solution
        //check the behavior of hashtable, not bad
        cout << hash->usedSize << ' ' << hash->nofElm << endl;
        //this is the output of a prerun, numbers are reachable final states finalS->order
        //amony them, due to left-right symmetry, only 4 solutions are interesting
/*
FOUND 2252 FOUND 2287 FOUND 7003 FOUND 7054 FOUND 7167 FOUND 7265 FOUND 22274 FOUND 23363
*/
        int foundS[] = {7003, 7054, 7265, 23363};
        cout << "Start SSSP\n";
        //start Single Source Shortest Path algorithm
        initS->steps = 0;
        //remember all final states, we'll delete hash to save mem
        State * f1 = hash->elm[2252];
        State * f2 = hash->elm[2287];
        State * f3 = hash->elm[7003];
        State * f4 = hash->elm[7054];
        State * f5 = hash->elm[7167];
        State * f6 = hash->elm[7265];
        State * f7 = hash->elm[22274];
        State * f8 = hash->elm[23363];
        delete hash;

        List<State *> *frontier = new List<State *>;
        //help to decide if a state is in frontier or not
        //other wise I have to write an efficient frontier->find(State *)
        char *inFrontier = new char[nofS];
        for (int i = 0; i < nofS; i++)
                inFrontier[i] = 0;
        
        frontier->append(initS);
        inFrontier[initS->order] = 1;
        while (!frontier->is_empty()) {
                State *v = frontier->pop();
                inFrontier[v->order] = 0;
                if (v->childS == NULL) continue;
                v->childS->reset();
                Item<State *> *tmpItem;
                while ((tmpItem = v->childS->retrieve()) != NULL) {
                        State *w = tmpItem->getval();
                        if (w->steps > (v->steps+1)) {
                                w->steps = v->steps+1;
                                w->parent = v;
                                if (! inFrontier[w->order]) {
                                        frontier->append(w);
                                        inFrontier[w->order] = 1;
                                }
                        }
                }
        }
        //print out nof Steps to reach each state
        cout << f1->steps << ' ' << f2->steps << ' ' << f3->steps << ' ' << f4->steps << ' '
                <<f5->steps << ' ' << f6->steps << ' ' << f7->steps << ' ' << f8->steps << endl;
        //output will be:97 100 100 97 81 81 85 85
        //print out the four interesting solutions
        printSSSP(initS, f6);
        cout << "***\n";
        printSSSP(initS, f8);
        cout << "***\n";
        printSSSP(initS, f4);
        cout << "***\n";
        printSSSP(initS, f3);
        return 0;
        //Happy Ending
}
